SystemOrganization addCategory: #'CS520-hm1'!

NamedBox subclass: #DancingBox
	instanceVariableNames: 'enableDancing danceSpeed danceRoutine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-hm1'!

!DancingBox commentStamp: 'CameronRieck 1/20/2013 11:52' prior: 0!
DancingBox represents a box that can dance.  It knows how fast to dance and what dance routine to do, but it doesn't know 
anything specific about the routine, which means you can swap out the routine any time you want and the DancingBox doesn't
have to change its behavior.

Instance Variables:
	danceSpeed	<Magnitude>
	danceRoutine	<DanceRoutine>!

!DancingBox methodsFor: 'dancing' stamp: 'CameronRieck 1/20/2013 13:41'!
danceWith: dancer
	| partnerPosition myStartPosition |
	danceRoutine setDancePartner: dancer.
	partnerPosition := dancer position.
	myStartPosition := partnerPosition + (0@100).
	self position: myStartPosition.
	self setDanceSpeed: 1.
	dancer startDancing.
	self startDancing.! !

!DancingBox methodsFor: 'initialization' stamp: 'CameronRieck 1/20/2013 13:06'!
initialize
	"initializes the dancing box"
	super initialize.
	super stopStepping.
	danceRoutine := DanceRoutineNil new.
	danceSpeed := 750.
	enableDancing := False.! !

!DancingBox methodsFor: 'dancing' stamp: 'CameronRieck 1/13/2013 23:06'!
setDanceRoutine: aDanceRoutine
	"sets the routine for the dancer"
	danceRoutine := aDanceRoutine.! !

!DancingBox methodsFor: 'initialization' stamp: 'CameronRieck 1/20/2013 13:03'!
setDanceSpeed: speed
	danceSpeed := speed.! !

!DancingBox methodsFor: 'dancing' stamp: 'CameronRieck 1/20/2013 13:08'!
startDancing
	"moves the box according to a dance move"
	enableDancing := True.
	super startStepping.! !

!DancingBox methodsFor: 'private' stamp: 'CameronRieck 1/20/2013 13:20'!
step
	"implements what needs to be done during a dance step"
	| danceMove |
	super step.
	(enableDancing = True) ifTrue: 
		[
		danceMove := danceRoutine nextMove.
		danceMove do:self.
		]! !

!DancingBox methodsFor: 'private' stamp: 'CameronRieck 1/13/2013 19:55'!
stepTime
	"comment stating purpose of message"
	^ danceSpeed.! !

!DancingBox methodsFor: 'dancing' stamp: 'CameronRieck 1/20/2013 13:42'!
stopDancing
	"stops dancing"
	enableDancing := False.
	self setDanceSpeed: 750.
	super stopStepping.! !

Object subclass: #DanceRoutine
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-hm1'!

!DanceRoutine commentStamp: 'CameronRieck 1/20/2013 12:35' prior: 0!
DanceRoutine represents a dance routine.  The specifics of a dance routine is left up to the subclasses.  All dance routines must implement the nextMove message.!

!DanceRoutine methodsFor: 'updating' stamp: 'CameronRieck 1/20/2013 12:27'!
nextMove
	"returns the next move to be performed in the routine"
	^ self subclassResponsibility.! !

DanceRoutine subclass: #DanceRoutineNil
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-hm1'!

!DanceRoutineNil commentStamp: 'CameronRieck 1/20/2013 12:07' prior: 0!
DanceRoutineNil represnts the empty dance routine.  No dance steps are performed with this routine.!

!DanceRoutineNil methodsFor: 'updating' stamp: 'CameronRieck 1/20/2013 12:29'!
nextMove
	^ (DanceStepMove new setMove:0@0).! !

DanceRoutine subclass: #DynamicDanceRoutine
	instanceVariableNames: 'dancePartner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-hm1'!

!DynamicDanceRoutine commentStamp: 'CameronRieck 1/20/2013 12:34' prior: 0!
DynamicDanceRoutine represents a dance routine that adapts to the state of the environment.  Typically, this will be the state of another dancer,
known as the partner.


Instance Variables:
	dancePartner	<DancingBox>!

DynamicDanceRoutine subclass: #DanceRoutineMimic
	instanceVariableNames: 'lastPositionOfPartner lastRotationOfPartner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-hm1'!

!DanceRoutineMimic commentStamp: 'CameronRieck 1/20/2013 12:07' prior: 0!
DanceRoutineMimic represents a dynamic dance routine that mimics the moves of another dancer.

Instance Variables:
	lastPositionOfPartner	<DancingBox>!

!DanceRoutineMimic methodsFor: 'updating' stamp: 'CameronRieck 1/20/2013 13:28'!
nextMove
	"Check position change of partner.  Move the same amount as change."
	| currentRotation changeInRotation currentPosition changeInPosition |
	currentRotation := dancePartner rotationDegrees.
	changeInRotation := currentRotation - lastRotationOfPartner.
	lastRotationOfPartner := currentRotation.
	(changeInRotation = 0) 
		ifFalse: [^ DanceStepTurn new setTurn: changeInRotation].
	
	currentPosition := dancePartner position.
	changeInPosition := currentPosition - lastPositionOfPartner.
	lastPositionOfPartner := currentPosition.
	^ DanceStepMove new setMove: changeInPosition.
	! !

!DanceRoutineMimic methodsFor: 'initialize-release' stamp: 'CameronRieck 1/20/2013 12:47'!
setDancePartner: dancer
	super setDancePartner: dancer.
	lastPositionOfPartner := dancer position.
	lastRotationOfPartner := dancer rotationDegrees.
	! !

!DynamicDanceRoutine methodsFor: 'initialize-release' stamp: 'CameronRieck 1/20/2013 12:26'!
setDancePartner: dancer
	dancePartner := dancer.! !

DanceRoutine subclass: #StaticDanceRoutine
	instanceVariableNames: 'routineList currentRoutine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-hm1'!

!StaticDanceRoutine commentStamp: 'CameronRieck 1/20/2013 12:33' prior: 0!
DanceRoutine represents a set of DanceSteps that define a "dance routine."  It is expected that one of the subclasses of DanceRoutine will be used.

Instance Variables:
	routineList	<Collection>
	currentRoutine	<Number>!

StaticDanceRoutine subclass: #DanceRoutineSpin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-hm1'!

!DanceRoutineSpin commentStamp: 'CameronRieck 1/20/2013 12:08' prior: 0!
DanceRoutineSpin represents a dance routine where the dancer moves around and performs a spin.!

!DanceRoutineSpin methodsFor: 'initialize-release' stamp: 'CameronRieck 1/13/2013 23:40'!
initialize
	super initialize.
	super addDanceStep: (DanceStepMove new setMove:0@20).
	super addDanceStep: (DanceStepMove new setMove:0@20).
	super addDanceStep: (DanceStepTurn new setTurn:45).
	super addDanceStep: (DanceStepTurn new setTurn:45).
	super addDanceStep: (DanceStepTurn new setTurn:(-45)).
	super addDanceStep: (DanceStepTurn new setTurn:(-45)).
	super addDanceStep: (DanceStepMove new setMove:0@(-20)).
	super addDanceStep: (DanceStepMove new setMove:0@(-20)).! !

StaticDanceRoutine subclass: #DanceRoutineSquare
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-hm1'!

!DanceRoutineSquare commentStamp: 'CameronRieck 1/20/2013 12:08' prior: 0!
DanceRoutineSquare represents the dance moves such that a dancer will move around in a square shape.!

!DanceRoutineSquare methodsFor: 'initialize-release' stamp: 'CameronRieck 1/13/2013 23:24'!
initialize
	super initialize.
	super addDanceStep: (DanceStepMove new setMove:0@20).
	super addDanceStep: (DanceStepMove new setMove:20@0).
	super addDanceStep: (DanceStepMove new setMove:0@(-20)).
	super addDanceStep: (DanceStepMove new setMove:(-20)@0).! !

StaticDanceRoutine subclass: #DanceRoutineTwist
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-hm1'!

!DanceRoutineTwist commentStamp: 'CameronRieck 1/20/2013 12:09' prior: 0!
DanceRoutineTwist represents the dance moves that move the dancer side to side and twists every now and then.!

!DanceRoutineTwist methodsFor: 'initialize-release' stamp: 'CameronRieck 1/13/2013 23:41'!
initialize
	super initialize.
	super addDanceStep: (DanceStepMove new setMove:20@0).
	super addDanceStep: (DanceStepMove new setMove:20@0).
	super addDanceStep: (DanceStepTurn new setTurn:45).
	super addDanceStep: (DanceStepMove new setMove:(-20)@0).
	super addDanceStep: (DanceStepMove new setMove:(-20)@0).
	super addDanceStep: (DanceStepTurn new setTurn:(-45)).! !

!StaticDanceRoutine methodsFor: 'adding' stamp: 'CameronRieck 1/20/2013 12:24'!
addDanceStep: danceStep
	"adds a dance step to the routine"
	routineList add:danceStep.! !

!StaticDanceRoutine methodsFor: 'initialize-release' stamp: 'CameronRieck 1/20/2013 12:31'!
initialize
	routineList := LinkedList new.
	currentRoutine := 1.! !

!StaticDanceRoutine methodsFor: 'updating' stamp: 'CameronRieck 1/20/2013 12:25'!
nextMove
	"returns the next move to be performed in the routine"
	| theNextMove |
	theNextMove := routineList at: currentRoutine.
	currentRoutine := currentRoutine + 1.
	(currentRoutine > routineList size) ifTrue:
	[
		currentRoutine := 1.
	].
	^ theNextMove.! !

Object subclass: #DanceStep
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-hm1'!

!DanceStep commentStamp: 'CameronRieck 1/20/2013 11:54' prior: 0!
DanceStep represents a movement that a dancer can perform.  At the moment, this includes a move and a turn.
The specific dance step tells the dancer to do the appropriate transformation.!

!DanceStep methodsFor: 'transformation' stamp: 'CameronRieck 1/20/2013 12:03'!
do: dancer
	"performs the dance step on the dancer"
	^ self subclassResponsibility.! !

DanceStep subclass: #DanceStepMove
	instanceVariableNames: 'movement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-hm1'!

!DanceStepMove commentStamp: 'CameronRieck 1/20/2013 12:36' prior: 0!
DanceStepMove represents a move transformation.

Instance Variables:
	movement	<Point>!

!DanceStepMove methodsFor: 'transformation' stamp: 'CameronRieck 1/13/2013 23:32'!
do: dancer
	"performs the movement"
	dancer move:movement.! !

!DanceStepMove methodsFor: 'initialize-release' stamp: 'CameronRieck 1/13/2013 23:18'!
setMove: newMovement
	"sets the move"
	movement := newMovement.
! !

DanceStep subclass: #DanceStepTurn
	instanceVariableNames: 'degreesToTurn'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-hm1'!

!DanceStepTurn commentStamp: 'CameronRieck 1/20/2013 12:36' prior: 0!
DanceStepTurn represents a turn transformation.

Instance Variables:
	degreesToTurn	<Integer>!

!DanceStepTurn methodsFor: 'transformation' stamp: 'CameronRieck 1/13/2013 23:31'!
do: dancer
	"performs the turn"
	dancer turn: degreesToTurn.! !

!DanceStepTurn methodsFor: 'initialize-release' stamp: 'CameronRieck 1/13/2013 23:30'!
setTurn: turnDegrees
	"sets the number of degrees to turn"
	degreesToTurn := turnDegrees.! !
